/*stripped down rats_encrypt rats_decrypt with no globals for possible lib*/


#include <stdio.h>
#include <sys/types.h>
#include <sys/errno.h>
#include <des.h>


/* takes plain text in vptr of length len and returns encrypted */
/* data in vptr. vptr must be big enough to be padded up to the */
/* 8 byte margin */ 

int rats_des_encrypt(char *vptr,int len, char *skey1,char *skey2)
{
  des_key_schedule sched1,sched2;
  char invec1[8];
  char invec2[8];
  char key1[8];
  char key2[8];

  unsigned char *mbuf;
  unsigned char *encd;
  unsigned char *e1;
  unsigned char *e2;

  memset(invec1,0,8);
  memset(invec2,0,8);
  des_string_to_key(skey1,key1);
  des_string_to_key(skey2,key2);
  des_key_sched(key1,sched1);
  des_key_sched(key2,sched2);
  
  

  mbuf = (unsigned char *)malloc(len);
  encd = (unsigned char *)malloc(len);
  e1 = (unsigned char *)malloc(len);
  e2 = (unsigned char *)malloc(len);
  test_malloc(mbuf, len, __LINE__);
  test_malloc(encd, len, __LINE__);
  test_malloc(e1, len, __LINE__);
  test_malloc(e2, len, __LINE__);


  memset(encd,0,len);
  memset(e1,0,len);
  memset(e2,0,len);
  memset(mbuf,0,len);

  array_copy(vptr,mbuf,len,len);

  des_cbc_encrypt(mbuf,encd,len,sched1,invec1,1);
  des_cbc_encrypt(encd,e1,len,sched2,invec2,0);
  des_cbc_encrypt(e1,e2,len,sched1,invec1,1);

  memset(vptr,0,len);

  array_copy(e2,vptr,len,len);

  free(mbuf);
  free(encd);
  free(e1);
  free(e2);

  return(0);
}


/* takes encrypted text in vptr of length len and returns clear */
/* data in vptr. vptr will always be big enough to hold the clear */
/* data as it will shrink or remain same on decrypt */

int rats_des_decrypt(void *vptr,char *skey1,char *skey2,int len)
{
  des_key_schedule sched1,sched2;
  char invec1[8];
  char invec2[8];
  char key1[8];
  char key2[8];

  unsigned char *mbuf;
  unsigned char *encd;
  unsigned char *e1;
  unsigned char *e2;

  mbuf = (unsigned char *)malloc(len);
  encd = (unsigned char *)malloc(len);
  e1 = (unsigned char *)malloc(len);
  e2 = (unsigned char *)malloc(len);
  test_malloc(mbuf, len, __LINE__);
  test_malloc(encd, len, __LINE__);
  test_malloc(e1, len, __LINE__);
  test_malloc(e2, len, __LINE__);


  memset(encd,0,len);
  memset(e1,0,len);
  memset(e2,0,len);
  memset(mbuf,0,len);
  memset(invec1,0,8);
  memset(invec2,0,8);

  des_string_to_key(skey1,key1);
  des_string_to_key(skey2,key2);
 
  des_key_sched(key1,sched1);
  des_key_sched(key2,sched2);


  array_copy(vptr,mbuf,len,len);
  
  des_cbc_encrypt(mbuf,encd,len,sched1,invec1,0);
  des_cbc_encrypt(encd,e1,len,sched2,invec2,1);
  des_cbc_encrypt(e1,e2,len,sched1,invec1,0);
 
  
  memset(vptr,0,len);

  /*e2 should be plain decoded text at this point*/
  /*this str function is safe*/
  array_copy(e2,vptr,len,len);
  /*strncpy(vptr,e2,strlen(e2)); */
  /*printf("vptr %s<p>",vptr);*/
  free(mbuf);
  free(encd);
  free(e1);
  free(e2);

  return(0);
}

/* copies arbitrary len bytes of data from source to target */
/* target should always be bigger then source, which is enforced */
/* by maxsize */

int array_copy(unsigned char *source,unsigned char *dest,int len,int maxsiz)
{
  int i;
  for(i=0;i<len && i<maxsiz;i++)
  {
    dest[i]=source[i];
  }
  return(i);
} 


unsigned char * rats_encrypt(char *text,char *key1,char *key2)
{
  unsigned char *inbuf;
  int templen;
  int inbufsiz;

  inbufsiz = strlen(text);


  /*make sure input buffer is big enough to handle 8-bit blocking*/
  /*and calculate output buffer size based off the magic blocking size*/
  if(inbufsiz%8)
    {
      templen = inbufsiz + (8-(inbufsiz%8));
    }
  else
    {
      templen = inbufsiz;
    }

  inbufsiz = templen;

  inbuf = (unsigned char *)malloc(inbufsiz);
  test_malloc(inbuf, inbufsiz, __LINE__);

  memset(inbuf,0,inbufsiz);

  array_copy(text,inbuf,strlen(text),inbufsiz);

  rats_des_encrypt(inbuf,inbufsiz,key1,key2);
  return(inbuf);

}

unsigned char * rats_decrypt(char *text,char *key1,char *key2,int length)
{
  unsigned char *inbuf,*outbuf;
  int inbufsiz,foo;

  inbufsiz = length;
  inbuf = (unsigned char *)malloc(inbufsiz);
  test_malloc(inbuf, inbufsiz, __LINE__);

  memset(inbuf,0,inbufsiz);

  array_copy(text,inbuf,length,inbufsiz);
  rats_des_decrypt(inbuf,key1,key2,inbufsiz);

  outbuf=(unsigned char *)malloc(inbufsiz+1);
  test_malloc(outbuf, inbufsiz+1, __LINE__);
  memset(outbuf,0,inbufsiz+1);

  foo=array_copy(inbuf,outbuf,inbufsiz,inbufsiz);
  return(outbuf);

}


int test_malloc(void *vptr, int size, int line) 
{
  if (vptr == NULL) {
    printf("malloc of %d bytes failed at line %d\n", size, line);
    exit(0);
  }
}
